.module       SINLUT;

.include <system.k>;
.include <midi.k>;

.global         handle_MIDI_controller;
.external       process_midi;
.external       rx_buf,tx_buf;
.global         main_loop,input_samples,init_params;
.external wave_sin;

{ signal processing variables here }

.var inputsig,phaseacc;

.const OVERS=4;
.var phaseoff[OVERS],magni[OVERS];
.init phaseoff: 0x0000,0x0000,0x0000,0x0000;
.init magni: 0x7fff,0x0000,0x0000,0x0000;


{ internal variables }
.var last_input;
.var last_output;
.init last_output:0;

{ parameters here }


{ command loop - check for incoming data on serial port }
main_loop:
        ar = dm (CHAR_WAITING_FLAG);    { check serial port for MIDI data }
        none = pass ar;
        if ne jump main_loop;
        call process_midi;        { if MIDI byte received }
        jump main_loop;




{ SPORT0 interrupt handler -- this is called at 44.1KHz }
input_samples:
        ena sec_reg;                { use shadow register bank }

        ax0 = dm(rx_buf +1);    { read a/d converters }
        ay0 = dm(rx_buf +2);    { leave left/right values in my0,my1 }

        ar  = ax0 + ay0;  { this a mono effect. }
        dm(inputsig) = ar;

        { use phase and magnitude }
        i0  = ^phaseoff;
        i1  = ^magni;
        l0  = 0;
        l1  = 0;
        l2  = 0;
        m0  = 0;
        m1  = 1;


        { init accumulators... }
        mr  = 0;
        ar  = 0;
        dm(phaseacc) = ar;

        cntr =  OVERS;

do lumpit until ce;

        ar  = dm(phaseacc);
        ay0 = dm(inputsig); { get input signal }
        ar  = ar + ay0;     { multiply via repeated adds }
        dm(phaseacc) = ar;  { accumulate it }

        ay0 = dm(i0,m1);  { get phase for this overtone }
        ar  = ar + ay0;   { addition does mod for us...}
        sr  = lshift ar by -8(lo);   { use ms 8-bits as index }
        ay0 = ^wave_sin;  { sine wave base }
        ar  = sr0 + ay0;   { index into it... }
        i2  = ar;        
        my0 = dm(i2,m0);  { lookup sinusoid }
        mx0 = dm(i1,m1);  { get magnitude for this overtone }
        
        mr  = mr + mx0*my0(ss); { accumulate components }
        if mv sat mr;
lumpit:
        nop;


        dm(tx_buf+ 1)=mr1;      { write d/a converters }
        dm(tx_buf+ 2)=mr1;      { write left/right values from my0, my1 }

        { save output value (for possible feedback) }
        dm(last_output)=mr1;
        
thatwasfun:
        rti;    





{ Here is where MIDI messages are used to set variables in your Effect! }
handle_MIDI_controller:
        ax1 = dm(midi_cont);
        ar  = dm(midi_val);
        ay1 =  1;none = ax1-ay1;if eq jump set_mag1;
        ay1 =  2;none = ax1-ay1;if eq jump set_phase1;
        ay1 =  3;none = ax1-ay1;if eq jump set_mag2;
        ay1 =  4;none = ax1-ay1;if eq jump set_phase2;
        ay1 =  5;none = ax1-ay1;if eq jump set_mag3;
        ay1 =  6;none = ax1-ay1;if eq jump set_phase3;
        ay1 =  7;none = ax1-ay1;if eq jump set_mag4;
        ay1 =  8;none = ax1-ay1;if eq jump set_phase4;
        rts;

set_mag1:
        sr  = lshift ar by 8(lo);
        dm(phaseoff+0) = sr0;
        rts;

set_phase1:
        sr  = lshift ar by 8(lo);
        dm(magni+0) = sr0;
        rts;

set_mag2:
        sr  = lshift ar by 8(lo);
        dm(phaseoff+1) = sr0;
        rts;

set_phase2:
        sr  = lshift ar by 8(lo);
        dm(magni+1) = sr0;
        rts;
set_mag3:
        sr  = lshift ar by 8(lo);
        dm(phaseoff+2) = sr0;
        rts;

set_phase3:
        sr  = lshift ar by 8(lo);
        dm(magni+2) = sr0;
        rts;

set_mag4:
        sr  = lshift ar by 8(lo);
        dm(phaseoff+3) = sr0;
        rts;

set_phase4:
        sr  = lshift ar by 8(lo);
        dm(magni+3) = sr0;
        rts;

{ do any dynamic memory/variable initializing you need to here }
init_params:
        rts;


.endmod;
